/******************************************************************************
* Module	:	Tiles --- Lay out tiles to solve a string ordering problem.
*
* Document	:	\section{Tiles}
*			:	
*			:	The problem to solve is how to lay out a number of tiles,
*			:	in a fixed size vector space and with no unfilled spots
*			:	in the result.
*			:	
*			:	There are a fixed number of tiles, such that each tile
*			:	can be represented by a vector whose contents are:
*			:	
*			:	tile = 'N' + N * Space + 'N'
*			:	
*			:	where Space is the ASCII space character, and 'N' is
*			:	the ASCII character that represents the number N.
*			:	
*			:	Tiles are laid on top of each other such that an 'N'
*			:	can only be placed within a space character.
*			:	
*			:	The vector into which tiles are placed satisfies:
*			:	
*			:	Vector = Space * (2 * N + 2)  iff zero is used, or
*			:	Vector = Space * (2 * N)      if  zero is not used.
*			:	
*			:	This program was put together to solve a problem given
*			:	to me by Mark Linroth: Mentor, Life Long Friend and
*			:	a Man of Many Parts.
*			:	
* EndDocum	:	
******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>
#include	<assert.h>

typedef	struct	AminoAcid_st
{
	int		offset;
	int		number;
	int		tail;
} AminoAcid_t;

typedef	struct	GeneString_st
{
	int				fitness;
	int				numberOfAcids;
	AminoAcid_t		**aminoAcids;
} GeneString_t;

typedef	struct	Population_st
{
	int				tileMaximum;
	int				populationSize;
	int				maximum;
	GeneString_t	**population;
} Population_t;

static	AminoAcid_t		*allTiles  = 0;
static	int				totalTiles = 0;
static	int				baseOne    = 0;
static	char			*vector	   = 0;
static	char			*score	   = 0;

showGeneString(
	GeneString_t	*geneString,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	register	int				k;
	auto		AminoAcid_t		*aminoAcid;

	/*	Does this string have any fitness at all?	*/
	if (geneString->fitness <= 0)
	{
		/*	Print a gene string.	*/
		printf(
			"%2d <*** BAD STRING ***>\n",
			geneString->fitness );
		return;
	}

	/*	Initialize vector.	*/
	memset(
		vector,
		' ',
		vectorSize );
	vector[ vectorSize ] = 0;
	memset(
		score,
		0,
		vectorSize );

	/*	For the number of amino acids in the gene, do:	*/
	for (i = 0; i < geneString->numberOfAcids; i++)
	{
		/*	Get an amino acid.	*/
		aminoAcid  = geneString->aminoAcids[ i ];

		/*	Is this amino acid an over specification?	*/
		if ( score[ aminoAcid->number ] )
			/*	Yes.  Just ignore this one.	*/
			continue;
		score[ aminoAcid->number ] = 1;

		/*	Yes.  For the number of atoms in the acid, do:	*/
		for (j = 0; j < 2; j++)
		{
			/*	Get atom number.	*/
			if (j == 0)
				k = aminoAcid->offset;
			else
				k = k + aminoAcid->number + 1;

			/*	Is this vector element (atom) empty?	*/
			if (vector[ k ] == ' ')
				/*	Put amino acid in place.	*/
				vector[ k ] = aminoAcid->number +
					((aminoAcid->number >= 10)
						?	'W'
						:	'0' );
			else
				vector[ k ] = '*';
		}
	}

	printf(
		"|%s|\n",
		vector );
}

void
showPopulation(
	Population_t	*population,
	int				vectorSize )
{
	register	int				i;
	auto		GeneString_t	*geneString;

	/*	We have now seeded the mGa population.
	*	
	*	Show the first 10 strings.
	*/
	printf(
		"*** Current Population:\n" );
	printf(
		"             l         2         3\n" );
	printf(
		"    123456789012345678901234567890\n" );
	for (i = 0; i < 1 && i < population->populationSize; i++)
	{
		/*	Get a gene string from the population.	*/
		geneString = population->population[ i ];

		/*	Show gene string.	*/
		showGeneString(
			geneString,
			vectorSize );
	}
}

void
calculateGeneStringFitness(
	GeneString_t	*geneString,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	register	int				k;
	auto		AminoAcid_t		*aminoAcid;

	/*	Clear the vector.	*/
	memset(
		vector,
		' ',
		vectorSize );
	memset(
		score,
		0,
		vectorSize );

	/*	Set gene string fitness to be zero.	*/
	geneString->fitness = 0;

	/*	For the number of amino acids, do:	*/
	for (i = 0; i < geneString->numberOfAcids; i++)
	{
		/*	Get amino acid.	*/
		aminoAcid = geneString->aminoAcids[ i ];

		/*	Is this amino acid an over specification?	*/
		if ( score[ aminoAcid->number ] )
			/*	Yes.  Just ignore this one.	*/
			continue;
		score[ aminoAcid->number ] = 1;

		/*	For the number of atoms in the acid, do:	*/
		for (j = 0; j < 2; j++)
		{
			/*	Get atom number.	*/
			if (j == 0)
				k = aminoAcid->offset;
			else
				k = k + aminoAcid->number + 1;

			/*	Is this vector element (atom) empty?	*/
			if (vector[ k ] == ' ')
			{
				/*	Put amino acid in place.	*/
				vector[ k ] = aminoAcid->number +
					((aminoAcid->number >= 10)
						?	'W'
						:	'0' );

				/*	Increase the fitness of this gene string
				*	by one.
				*/
				geneString->fitness++;
			}
			else
			{
				/*	No.  The fitness of this gene
				*	string is reduced by two.
				*/
				geneString->fitness -= 2;
			}
		}
	}

	/*	Have we found the string we are looking for?	*/
	if (geneString->fitness == vectorSize)
	{
		showGeneString(
			geneString,
			vectorSize );
		exit(
			0 );
	}
}

AminoAcid_t	*
createRandomAminoAcid(
	void )
{
	auto	AminoAcid_t		*aminoAcid;

	/*	Choose a random tile from the set of all tiles.	*/
	aminoAcid = allTiles +
		random(
			) % totalTiles;

	/*	Return the gene string.	*/
	return aminoAcid;
}

GeneString_t	*
createSingleRandomAminoAcidGeneString(
	int		vectorSize,
	int		tileMaximum )
{
	auto	AminoAcid_t		*aminoAcid;
	auto	GeneString_t	*geneString;

	/*	Can we allocate a gene string?	*/
	if ( (geneString =
		calloc(
			1,
			sizeof ( GeneString_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Can we allocate a list to store amino acids in?	*/
	if ( (geneString->aminoAcids =
		calloc(
			1,
			sizeof ( AminoAcid_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Get a single random amino acid.	*/
	aminoAcid =
		createRandomAminoAcid(
			);
			
	/*	Create gene string.	*/
	geneString->aminoAcids[ 0 ] = aminoAcid;

	/*	Initialize gene string.	*/
	geneString->numberOfAcids = 1;
	geneString->fitness       = 1;

	/*	Return the gene string.	*/
	return geneString;
}

GeneString_t	*
duplicateGeneString(
	GeneString_t	*sourceString )
{
	register	int				i;
	auto		GeneString_t	*geneString;

	/*	Can we allocate a gene string?	*/
	if ( (geneString =
		calloc(
			1,
			sizeof ( GeneString_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Initialize gene string.	*/
	geneString->numberOfAcids = sourceString->numberOfAcids;

	/*	Can we allocate a list to store amino acids in?	*/
	if ( (geneString->aminoAcids =
		calloc(
			sourceString->numberOfAcids + 1,
			sizeof ( AminoAcid_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Copy amino acids.	*/
	for (i = 0; i < sourceString->numberOfAcids; i++)
		geneString->aminoAcids[ i ] = sourceString->aminoAcids[ i ];

	/*	Return the gene string.	*/
	return geneString;
}

GeneString_t	*
createMessyString(
	AminoAcid_t		*aminoAcid )
{
	auto	GeneString_t	*geneString;

	/*	Can we allocate a gene string?	*/
	if ( (geneString =
		calloc(
			1,
			sizeof ( GeneString_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Can we allocate a list to store amino acids in?	*/
	if ( (geneString->aminoAcids =
		calloc(
			1,
			sizeof ( AminoAcid_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Create gene string.	*/
	geneString->aminoAcids[ 0 ] = aminoAcid;

	/*	Initialize gene string.	*/
	geneString->numberOfAcids = 1;
	geneString->fitness       = 1;

	/*	Return the gene string.	*/
	return geneString;
}

void
mutateIndividuals(
	Population_t	*population,
	int				percentage,
	int				vectorSize )
{
	register	int				i;
	auto		int				acid;
	auto		int				individual;
	auto		int				numberOfMutations;
	auto		AminoAcid_t		*aminoAcid;
	auto		GeneString_t	*geneString;

	/*	Determine number of individuals to mutate.	*/
	numberOfMutations = (population->populationSize * percentage) / 100;

	/*	For the number of individuals to mutate, do:	*/
	for (i = 0; i < numberOfMutations; i++)
	{
		/*	Choose a random individual.	*/
		individual =
			random(
				) % population->populationSize;
		geneString = population->population[ individual ];

		/*	Does this gene string have at least two amino
		*	acids in it?
		*/
		if (geneString->numberOfAcids < 2)
			/*	No.	Mutate existing acid.	*/
			acid = 0;
		else
			/*	Choose a random amino acid in the gene's
			*	string of amino acids.
			*/
			acid =
				random(
					) % geneString->numberOfAcids;

		/*	Get single random amino acid.	*/
		aminoAcid =
			createRandomAminoAcid(
				);

		/*	Destroy existing amino acid.	*/
		geneString->aminoAcids[ acid ] = aminoAcid;
	}
}

GeneString_t	*
fuseTwoGeneStrings(
	GeneString_t	*one,
	GeneString_t	*two,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	auto		int				numberOfAcids;
	auto		AminoAcid_t		*aminoAcid;
	auto		GeneString_t	*geneString;

	/*	Calculate the size of the new amino acid list.	*/
	numberOfAcids = one->numberOfAcids + two->numberOfAcids;

	/*	Can we allocate a gene string?	*/
	if ( (geneString =
		calloc(
			1,
			sizeof ( GeneString_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Save number of amino acids.	*/
	geneString->numberOfAcids = numberOfAcids;

	/*	Can we allocate a list to store amino acids in?	*/
	if ( (geneString->aminoAcids =
		calloc(
			geneString->numberOfAcids,
			sizeof ( AminoAcid_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Concatenate gene strings.	*/
	i = 0;
	for (j = 0; j < one->numberOfAcids; j++, i++)
		geneString->aminoAcids[ i ] = one->aminoAcids[ j ];
	for (j = 0; j < two->numberOfAcids; j++, i++)
		geneString->aminoAcids[ i ] = two->aminoAcids[ j ];

	/*	Return new gene string.	*/
	return geneString;
}

void
fusion(
	Population_t	*population,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	auto		int				newSize;
	auto		GeneString_t	*geneString;
	auto		GeneString_t	*one;
	auto		GeneString_t	*two;

	/*	Get new population size.	*/
	newSize = population->populationSize << 1;

	/*	For the number of gene strings, do:	*/
	for (i = 0; i < population->populationSize; i++)
	{
		/*	Get first gene string.	*/
		j =
			random(
				) % population->populationSize;
		one = population->population[ j ];

		/*	Get second gene string.	*/
		j =
			random(
				) % population->populationSize;
		two = population->population[ j ];

		/*	Fuse the two gene strings.	*/
		geneString =
			fuseTwoGeneStrings(
				one,
				two,
				vectorSize );

		/*	Add to population.	*/
		population->population[ population->populationSize + i ] = geneString;
	}

	/*	Save new population size.	*/
	population->populationSize = newSize;
}

void
crossOverTwoGeneStrings(
	Population_t	*population,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	auto		int				breakPoint;
	auto		int				minimum;
	auto		AminoAcid_t		*aminoAcid;
	auto		GeneString_t	*one;
	auto		GeneString_t	*two;
	auto		GeneString_t	*newOne;
	auto		GeneString_t	*newTwo;

	/*	Choose two random gene strings.	*/
	i =
		random(
			) % population->populationSize;
	one = population->population[ i ];
	i =
		random(
			) % population->populationSize;
	two = population->population[ i ];

	/*	Minimum.	*/
	minimum =
		(one->numberOfAcids < two->numberOfAcids)
			?	one->numberOfAcids
			:	two->numberOfAcids ;

	/*	Chose a random breaking point.	*/
	breakPoint =
		random(
			) % minimum;

	/*	Can we allocate a new gene string?	*/
	if ( (newOne =
		calloc(
			1,
			sizeof ( GeneString_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Initialize new gene string one.	*/
	newOne->numberOfAcids = breakPoint + (two->numberOfAcids - breakPoint);

	/*	Can we allocate new amino acid list for gene string one?	*/
	if ( (newOne->aminoAcids =
		calloc(
			newOne->numberOfAcids,
			sizeof ( AminoAcid_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	For the number of amino acids before the break point, do:	*/
	for (i = 0; i < breakPoint; i++)
		/*	Copy amino acids to new gene string.	*/
		newOne->aminoAcids[ i ] = one->aminoAcids[ i ];

	/*	For the number of amino acids after the break point, do:	*/
	for (j = breakPoint; j < two->numberOfAcids; i++, j++)
		/*	Copy amino acids to new gene string.	*/
		newOne->aminoAcids[ i ] = two->aminoAcids[ j ];

	/*	Can we allocate a new gene string?	*/
	if ( (newTwo =
		calloc(
			1,
			sizeof ( GeneString_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Initialize new gene string two.	*/
	newTwo->numberOfAcids = breakPoint + (one->numberOfAcids - breakPoint);

	/*	Can we allocate new amino acid list for gene string two?	*/
	if ( (newTwo->aminoAcids =
		calloc(
			newTwo->numberOfAcids,
			sizeof ( AminoAcid_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	For the number of amino acids before the break point, do:	*/
	for (i = 0; i < breakPoint; i++)
		/*	Copy amino acids to new gene string.	*/
		newTwo->aminoAcids[ i ] = two->aminoAcids[ i ];

	/*	For the number of amino acids after the break point, do:	*/
	for (j = breakPoint; j < one->numberOfAcids; i++, j++)
		/*	Copy amino acids to new gene string.	*/
		newTwo->aminoAcids[ i ] = one->aminoAcids[ j ];

	/*	Add new gene strings to population.	*/
	population->population[ population->populationSize++ ] = newOne;
	population->population[ population->populationSize++ ] = newTwo;
}

void
createNewGeneStringsFromCrossOver(
	Population_t	*population,
	int				percentage,
	int				vectorSize )
{
	register	int		i;
	auto		int		newSize;

	/*	Get new population size.	*/
	newSize = population->populationSize +
		(percentage * population->populationSize) / 100;
	newSize +=
		(newSize & 1)
			?	1
			:	0 ;

	/*	Create new gene strings from cross over.	*/
	for (i = population->populationSize; i < newSize; i += 2)
		/*	Do cross over between two gene strings.	*/
		crossOverTwoGeneStrings(
			population,
			vectorSize );

	/*	Save new size.	*/
	population->populationSize = newSize;
}

int
compareTwoGeneStrings(
	GeneString_t	**one,
	GeneString_t	**two )
{
	return (*two)->fitness - (*one)->fitness;
}

void
cullThePopulation(
	Population_t	*population,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	auto		GeneString_t	*geneString;

	/*	For the size of the population, do:	*/
	for (i = 0; i < population->populationSize; i++)
	{
		/*	Get the gene string.	*/
		geneString = population->population[ i ];

		/*	Calculate fitness of new string.	*/
		calculateGeneStringFitness(
			geneString,
			vectorSize );
	}

	/*	Sort population in reverse order.	*/
	qsort(
		population->population,
		population->populationSize,
		sizeof ( GeneString_t * ),
		(int (*)(const void *, const void *)) compareTwoGeneStrings );

	/*	Is the population big enough to start culling?	*/
	if (population->populationSize < population->maximum)
		return;

	/*	Dispose of all gene strings above the maximum.	*/
	for (i = population->maximum; i < population->populationSize; i++)
	{
		/*	Get a pointer to the gene string.	*/
		geneString = population->population[ i ];

		/*	Free the amino acid list.	*/
		free(
			geneString->aminoAcids );

		/*	Free the gene string.	*/
		free(
			geneString );

		/*	Zero out population list entry.	*/
		population->population[ i ] = 0;
	}

	/*	Set population size to maximum.	*/
	population->populationSize = population->maximum;
}

void
seedPopulationWithSingleAcidGeneStrings(
	Population_t	*population,
	int				percentage,
	int				vectorSize )
{
	register	int				i;
	auto		int				newSize;
	auto		GeneString_t	*geneString;

	/*	Get new population size.	*/
	newSize = population->populationSize +
		(percentage * population->populationSize) / 100;

	/*	For the number of strings in the initial population, do:	*/
	for (i = population->populationSize; i < newSize; i++)
	{
		/*	Create a single amino acid gene string.	*/
		geneString =
			createSingleRandomAminoAcidGeneString(
				vectorSize,
				population->tileMaximum );

		/*	Add gene string to population.	*/
		population->population[ i ] = geneString;
	}

	/*	Save new size.	*/
	population->populationSize = newSize;
}

void
addOrDeleteAminoAcid(
	Population_t	*population,
	int				percentage,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	auto		int				newSize;
	auto		GeneString_t	*copy;
	auto		GeneString_t	*geneString;
	auto		AminoAcid_t		*aminoAcid;

	/*	Get new population size.	*/
	newSize = population->populationSize +
		(percentage * population->populationSize) / 100;
	assert(
		newSize < 3 * population->maximum );

	/*	For the number of strings in the initial population, do:	*/
	for (i = population->populationSize; i < newSize; i++)
	{
		/*	Choose a random gene string.	*/
		geneString =
			population->population[
				random(
					) % population->populationSize ];

		/*	Copy gene string.	*/
		copy = 
			duplicateGeneString(
				geneString );

		/*	Insert, or delete?	*/
		if (
			random(
				) & 1 )
		{
			/*	Choose an insertion point.	*/
			j =
				random(
					) % (copy->numberOfAcids + 1);

			/*	Insert.  Do we need to move elements up
			*	to make room?
			*/
			if (j < copy->numberOfAcids)
				/*	Yes.	*/
				memmove(
					copy->aminoAcids + j + 1,
					copy->aminoAcids + j,
					sizeof ( AminoAcid_t * ) *
						(copy->numberOfAcids - j) );

			/*	Insert new amino acid.	*/
			copy->aminoAcids[ j ] =
				createRandomAminoAcid(
					);

			/*	Increment the number of acids.	*/
			copy->numberOfAcids++;
		}
		else if (copy->numberOfAcids > 1)
		{
			/*	Choose a deletion point.	*/
			j =
				random(
					) % copy->numberOfAcids;

			/*	Do we need to move elements down?	*/
			if (j < --copy->numberOfAcids)
				/*	Yes.	*/
				memmove(
					copy->aminoAcids + j,
					copy->aminoAcids + j + 1,
					sizeof ( AminoAcid_t * ) *
						(copy->numberOfAcids - j) );
		}

		/*	Calculate fitness.	*/
		calculateGeneStringFitness(
			copy,
			vectorSize );

		/*	Add gene string to population.	*/
		population->population[ i ] = copy;
	}

	/*	Save new size.	*/
	population->populationSize = newSize;
}

void
reorderGeneStrings(
	Population_t	*population,
	int				percentage,
	int				vectorSize )
{
	register	int				i;
	register	int				j;
	register	int				k;
	auto		int				newSize;
	auto		GeneString_t	*geneString;
	auto		AminoAcid_t		*aminoAcid;

	/*	Get new population size.	*/
	newSize = population->populationSize +
		(percentage * population->populationSize) / 100;
	assert(
		newSize < 3 * population->maximum );

	/*	For the number of strings in the initial population, do:	*/
	for (i = population->populationSize; i < newSize; i++)
	{
		/*	Choose a random gene string.	*/
		geneString =
			population->population[
				random(
					) % population->populationSize ];

		/*	Duplicate the gene string.	*/
		geneString =
			duplicateGeneString(
				geneString );

		/*	Get two locations to swap.	*/
		j =
			random(
				) % geneString->numberOfAcids;
		k =
			random(
				) % geneString->numberOfAcids;

		/*	Get pointer to one acid.	*/
		aminoAcid                   = geneString->aminoAcids[ j ];
		geneString->aminoAcids[ j ] = geneString->aminoAcids[ k ];
		geneString->aminoAcids[ k ] = aminoAcid;

		/*	Add gene string to population.	*/
		population->population[ i ] = geneString;
	}

	/*	Save new size.	*/
	population->populationSize = newSize;
}

void
reproduceAnotherGeneration(
	Population_t	*population,
	int				vectorSize )
{
	/*	Mutate some individuals.	*/
	mutateIndividuals(
		population,
		10,
		vectorSize );

	/*	Do cross over.	*/
	createNewGeneStringsFromCrossOver(
		population,
		10,
		vectorSize );

	/*	Add some new amino acids.	*/
	addOrDeleteAminoAcid(
		population,
		10,
		vectorSize );

	/*	Reorder some strings.	*/
	reorderGeneStrings(
		population,
		10,
		vectorSize );

	/*	Cull the population.	*/
	cullThePopulation(
		population,
		vectorSize );
}

int
main(
	int		argc,
	char	**argv )
{
	register	int				i;
	register	int				j;
	register	int				k;
	auto		Population_t	*population;
	auto		int				vectorSize;
	auto		AminoAcid_t		*aminoAcid;
	auto		GeneString_t	*geneString;

	/*	Seed random number generator from clock.	*/
	srandom(
		time(
			0 ) );

	/*	Do we have enough command line parameters?	*/
	if (argc < 4)
	{
		/*	No.  Print error and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - syntax is\n"
			"tiles <maximum population size> <maximum tile size> <base one>\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Can we allocate an initial population structure?	*/
	if ( (population =
		calloc(
			1,
			sizeof ( Population_t ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Get command line parameters.	*/
	population->maximum     =
		atoi(
			argv[ 1 ] );
	population->tileMaximum =
		atoi(
			argv[ 2 ] );
	baseOne                 =
		atoi(
			argv[ 3 ] );
	baseOne                 =
		( baseOne )
			?	1
			:	0 ;

	/*	Calculate the vector size.	*/
	vectorSize = population->tileMaximum * 2 + 2 * (1 - baseOne);

	/*	Can we allocate a vector?	*/
	if ( (vector =
		calloc(
			1,
			vectorSize + 1 )) == 0 )
	{
		/*	No.  Print error and exit.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Can we allocate a score board for over specification?	*/
	if ( (score =
		calloc(
			1,
			vectorSize )) == 0 )
	{
		/*	No.  Print error and exit.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	Can we allocate an initial population?	*/
	if ( (population->population =
		calloc(
			population->maximum * 3,
			sizeof ( GeneString_t * ) )) == 0 )
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	For all tiles, do:	*/
	totalTiles = 0;
	for (i = baseOne; i <= population->tileMaximum; i++)
		/*	For all possible combinations of tiles, do:	*/
		totalTiles += vectorSize - i - 1;

	/*	Allocate enough memory for all tiles.	*/
	if ( (allTiles =
		calloc(
			totalTiles + 2,
			sizeof ( AminoAcid_t ) )) == 0)
	{
		/*	No.  Print error, and exit program.	*/
		fprintf(
			stderr,
			"%s %d : Error - out of memory.\n",
			__FILE__,
			__LINE__ );
		exit(
			1 );
	}

	/*	For all tiles, do:	*/
	for (i = baseOne, k = 0; i <= population->tileMaximum; i++)
	{
		/*	For all possible combinations of tiles, do:	*/
		for (j = 0; j < vectorSize - i - 1; j++, k++)
		{
			/*	Create tile.	*/
			allTiles[ k ].offset = j;
			allTiles[ k ].number = i;
			allTiles[ k ].tail   = vectorSize - j - i - 2;

			/*	Create messy GA gene string.	*/
			geneString =
				createMessyString(
					allTiles + k );

			/*	Append to population.	*/
			population->population[ population->populationSize++ ] =
				geneString;
		}
	}

	/*	Determine the best number of fusions.	*/
	for (k = 0; (1 << k) <= population->tileMaximum; k++)
		;
	k += 2;

	/*	Fuse, mutate, cull and repeat a few times.	*/
	for (i = 0; i < k; i++)
	{
		/*	Fuse genes.	*/
		fusion(
			population,
			vectorSize );

		/*	Mutate some individuals.	*/
		mutateIndividuals(
			population,
			15,
			vectorSize );

		/*	Cull the population.	*/
		cullThePopulation(
			population,
			vectorSize );
	}

	/*	Reproduce another generation.	*/
	for (i = 0; i < 4000; i++)
		reproduceAnotherGeneration(
			population,
			vectorSize );

	printf(
		">>> Giving up!\n" );
}
